unit DS030U1;
(*Written with D2
  Basic demo of read/ write of DS2408, 8 bit I/O chip

  MicroLan is an alias for 1-Wire. The latter is used more
  commonly at the sites I see, but gives search engines and
  abbreviators problems. (OW: "One Wire"... or is that Zero W?)

  In this, DML, for Dallas MicroLan, is used extensively
  to prefix names so that this can be exported to other
  environments with minimal name clash problems. Where a
  name does not include DML, the object is probably not
  central to what needs to be exported.

  Created by TK Boyd. See program's copyright message.
  If you use this material, a credit and citation of
  http://sheepdogsoftware.co.uk would be welcome.

  DS030 started Easter Day, 11 Apr, 2004, SX

  To Do:
  The chip provides internal monitoring of the inputs. I.e.
  it can "remember" if there's been a transition on any
  line since the last time you looked. Accessing and clearing
  the relevant register would not be hard, or hard to add
  to this demo.

  The chip also provides for the normal-for-most 1-Wire chips
  "Conditional Searches". This functionality should be written up.
  *)

(*DS030 Derived from DS029... which had these comments...
  DS029 needs MUCH work on error codes...(DS030 has had much
  of the required work)*)

(*CONSIDER: CAN Delphi 1 cope with cardinal type?*)

(*Demo code showing how to read the VAD register from a DS2438
  Battery ID/ Monitor chip on a Dallas Semiconductor MicroLan(T)
  or 1-Wire(T) system.

(DS029)Started (for second time) 10 August 03 in connection
with GW weather stn*)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, iBTMEXPW, ExtCtrls;
  (*iBTMEXPW.PAS is supplied by Dallas as part of the TMEX SDK
    There are several ways to make what is needed available
    to Delphi. The easiest is simply to keep a copy of iBTMEXPW.pas
    in the folder with your .pas file*)

const
  ver='12 April 04';
  kDMLRomIndexMax=4;(*Determines maximum number of
     devices that can be accessed. If 4, 5 devices
     possible. Unused elements waste some memory*)
     clOn=clLime;
     clOff=clGray;

type
  TDS030f1 = class(TForm)
    buQuit: TButton;
    Label3: TLabel;
    Label4: TLabel;
    laDMLTKBErr: TLabel;
    laDMLDalErr: TLabel;
    buReadPIO: TButton;
    paPIOEnables: TPanel;
    laPIO: TLabel;
    laTxtPIOEnables: TLabel;
    buSafe: TButton;
    cbPIOEnabled7: TCheckBox;
    cbPIOEnabled6: TCheckBox;
    cbPIOEnabled5: TCheckBox;
    cbPIOEnabled4: TCheckBox;
    cbPIOEnabled3: TCheckBox;
    cbPIOEnabled2: TCheckBox;
    cbPIOEnabled1: TCheckBox;
    cbPIOEnabled0: TCheckBox;
    laTxtPIOEnables2: TLabel;
    paStates: TPanel;
    shOnOff7: TShape;
    shOnOff6: TShape;
    shOnOff5: TShape;
    shOnOff4: TShape;
    shOnOff3: TShape;
    shOnOff2: TShape;
    shOnOff1: TShape;
    shOnOff0: TShape;
    Label5: TLabel;
    laTxtInputs: TLabel;
    cbOutput7: TCheckBox;
    cbOutput6: TCheckBox;
    cbOutput5: TCheckBox;
    cbOutput4: TCheckBox;
    cbOutput3: TCheckBox;
    cbOutput2: TCheckBox;
    cbOutput1: TCheckBox;
    cbOutput0: TCheckBox;
    Label6: TLabel;
    buWriteToOutput: TButton;
    laTxtRememeber: TLabel;
    laTxtRemeber2: TLabel;
    buLoop: TButton;
    laTextLoop: TLabel;
    Timer1: TTimer;
    latxtFlaw: TLabel;
    laTxtFlaw2: TLabel;
    laTxtFlaw3: TLabel;
    buCopyright: TButton;
    buHelp: TButton;
    procedure buQuitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure buReadPIOClick(Sender: TObject);
    procedure buSafeClick(Sender: TObject);
    procedure cbPIOEnabled7Click(Sender: TObject);
    procedure cbPIOEnabled6Click(Sender: TObject);
    procedure cbPIOEnabled5Click(Sender: TObject);
    procedure cbPIOEnabled4Click(Sender: TObject);
    procedure cbPIOEnabled3Click(Sender: TObject);
    procedure cbPIOEnabled2Click(Sender: TObject);
    procedure cbPIOEnabled1Click(Sender: TObject);
    procedure cbPIOEnabled0Click(Sender: TObject);
    procedure buWriteToOutputClick(Sender: TObject);
    procedure buLoopClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure buCopyrightClick(Sender: TObject);
    procedure buHelpClick(Sender: TObject);
    procedure cbOutput0Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    rTmp:single;(*type "real" not recommended*)
    bTmp, bCycles:byte;
    sTmp,sTKBErrReturned:string;
    iTmp,iDalErrReturned,iD0,iD1,iD2,iD3,iD4,iD5,iD6,iD7,iD8:integer;
      (*Yes: iD0 to iD8, not 7. Could be done more elegantly... but simple is best?*)
    bValueSupplied,bMaskSupplied:byte;
    bDMLRomIndexLast:byte;
    saDMLRomID:array [0..kDMLRomIndexMax] of string[16];
    (*While this example accesses only one chip, it is
      written so that extending it to more is easy.
      saDMLRomID[0] would hold 'A20000000547FB10' if
      the first chip's id were $A20000000547FB10. The
      address is given in the form returned by the
      iButtonViewer (software included in SDK), i.e.
      the right hand byte is the device ID byte.*)
    baDMLRomID:array [0..kDMLRomIndexMax,0..7] of byte;
    (*baDMLRomID[2,0] holds chip 2's first id byte*)
    iDMLPortNum,iDMLAdapterType:integer;
    (*Beware: I have before, and may still, have been confused
    over the type expectations of the TMEX interface. I THINK
    that Dallas calls a signed 8bit number a "byte" whereas
    in Delphi-speak a byte is an UNsigned 8 bit number. IF all
    of this program is done right, AND you never have to look
    at the Dallas documentation (seems unlikely!), then you'll
    be okay. If you spot errors on my part in translating from
    the Dallas to the Delphi, please get in touch?
    ...and... just to add to the fun... I think I remember some
    issues regarding changed type definitions (or at least
    types of type available) between the 16 bit Delphi (Delphi ver.1)
    and later versions. That sounds unlikely, so I'm probably
    mis-remembering... but watch out for whatever caught me that
    registered in my memory thus.

    From SDK help file: (except first column)...
    Delphi name Dallas name   Desc
    integer     short         16-bit signed
    longint     long          32-bit signed
    shortint    char          8-bit signed
    byte        unsigned char 8-bit unsigned

    Checked, should be exactly right types:
    iDMLPortNum, iDMLAdapterType,
    liDMLSHandle
    siaDMLRom
    baDMLRom
    iDMLTmpResult... should be i for TMSetup, Access
       and TouchByte, at least. Would be mad to be
       inconsistent... but let me know if one of the
       others returns something different, other than
       TMExtendedStartSession

    *)
    iDMLTmpResult:integer;
    (*iDMLTmpResult is a 'scratch' variable, never used for
         long-term storage of any value.*)
    liDMLSHandle:longint;
    baDMLStateBuffer:array[0..15361] of byte;
      (*15360 might do, but for one byte, I covered
       the possiblilty that I was misunderstanding
       something in the Dallas docs*)
    siaDMLRom:array[0..8] of smallint;(*Yes: 9 bytes.
       the last isn't used every time, but I think
       it IS used sometimes.

       A curiosity: The type MUST BE SMALLINT...
       I accidentally had this as byte and got
       GPFaults during TMRom call. (Problem
       stuffing $FF into an element?)*)
    cardTmp:cardinal;(*The cardinal type doesn't appear
       in the Delphi 1 helpfiles, but it doesn't cause
       error during compile. Believed to be a 32 bit
       unsigned type. IS BAD IDEA!
       qexpand the cardinal issue*)

    function bState(cTmp:TCheckBox):byte;

    procedure DMLInit;
    procedure DMLGetPIOLogicState(bIndex:byte;
         var bAns:byte;
         var sTKBErr:string;
         var iDalErr:integer);

    procedure DMLWriteToEnabledOutputs
         (bIndex,bValue,bMask:byte;
         var sTKBErr:string;
         var iDalErr:integer);


  end;

var
  DS030f1: TDS030f1;

implementation

{$R *.DFM}

procedure TDS030f1.cbOutput0Click(Sender: TObject);
begin
buWriteToOutputClick(buQuit);
//buQuit ignored in above, but kept procedure happy.
end;

procedure TDS030f1.buCopyrightClick(Sender: TObject);
begin
showmessage('This material is copyright April 2004 by TK Boyd, '+
'of Sheepdog Software (tm). Permission is granted '+
'for non-commercial use. Please cidte source and '+chr(13)+chr(13)+
'       http://sheepdogsoftware.co.uk');
end;

procedure TDS030f1.buHelpClick(Sender: TObject);
begin
application.helpcommand(help_contents,0);
end;

function TDS030f1.bState(cTmp:TCheckBox):byte;
begin
if cTmp.checked then bState:=1 //no ;
else bState:=0;
end;

procedure TDS030f1.buQuitClick(Sender: TObject);
begin
application.terminate;
end;

procedure TDS030f1.FormCreate(Sender: TObject);
begin
DMLInit;
application.title:='DS030';(*Taskbar title*)
Application.HelpFile:='DS030.hlp';
caption(*window title bar*):='DS030, version '+ver+' from http://sheepdogsoftware.co.uk';
shOnOff7.brush.color:=clOff;shOnOff6.brush.color:=clOff;
shOnOff5.brush.color:=clOff;shOnOff4.brush.color:=clOff;
shOnOff3.brush.color:=clOff;shOnOff2.brush.color:=clOff;
shOnOff1.brush.color:=clOff;shOnOff0.brush.color:=clOff;
bCycles:=0;
end;

procedure TDS030f1.buReadPIOClick(Sender: TObject);
begin
buReadPIO.enabled:=false;
buWriteToOutput.enabled:=false;
DMLGetPIOLogicState(0,bTmp,sTKBErrReturned,iDalErrReturned);
laPIO.caption:='$'+inttohex(bTmp,2);
if (bTmp and $80>0) then shOnOff7.brush.color:=clOn // no ;
  else shOnOff7.brush.color:=clOff;
if (bTmp and $40>0) then shOnOff6.brush.color:=clOn // no ;
  else shOnOff6.brush.color:=clOff;
if (bTmp and $20>0) then shOnOff5.brush.color:=clOn // no ;
  else shOnOff5.brush.color:=clOff;
if (bTmp and $10>0) then shOnOff4.brush.color:=clOn // no ;
  else shOnOff4.brush.color:=clOff;
if (bTmp and $8>0) then shOnOff3.brush.color:=clOn // no ;
  else shOnOff3.brush.color:=clOff;
if (bTmp and $4>0) then shOnOff2.brush.color:=clOn // no ;
  else shOnOff2.brush.color:=clOff;
if (bTmp and $2>0) then shOnOff1.brush.color:=clOn // no ;
  else shOnOff1.brush.color:=clOff;
if (bTmp and $1>0) then shOnOff0.brush.color:=clOn // no ;
  else shOnOff0.brush.color:=clOff;
{laVAD.caption:=inttostr(cardTmp);{'FAKED 14';}
laDMLTKBErr.caption:=sTKBErrReturned;
laDMLDalErr.caption:=inttostr(iDalErrReturned);
buReadPIO.enabled:=true;
buWriteToOutput.enabled:=true;
end;

procedure TDS030f1.buWriteToOutputClick(Sender: TObject);
begin
buReadPIO.enabled:=false;
buWriteToOutput.enabled:=false;
bMaskSupplied:=bState(cbPIOEnabled7);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled6);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled5);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled4);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled3);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled2);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled1);
bMaskSupplied:=bMaskSupplied*2+bState(cbPIOEnabled0);

bValueSupplied:=bState(cbOutput7);
bValueSupplied:=bValueSupplied*2+bState(cbOutput6);
bValueSupplied:=bValueSupplied*2+bState(cbOutput5);
bValueSupplied:=bValueSupplied*2+bState(cbOutput4);
bValueSupplied:=bValueSupplied*2+bState(cbOutput3);
bValueSupplied:=bValueSupplied*2+bState(cbOutput2);
bValueSupplied:=bValueSupplied*2+bState(cbOutput1);
bValueSupplied:=bValueSupplied*2+bState(cbOutput0);
DMLWriteToEnabledOutputs(0,bValueSupplied,bMaskSupplied,sTKBErrReturned,iDalErrReturned);
laDMLTKBErr.caption:=sTKBErrReturned;
laDMLDalErr.caption:=inttostr(iDalErrReturned);
buReadPIO.enabled:=true;
buWriteToOutput.enabled:=true;
end;

procedure TDS030f1.buLoopClick(Sender: TObject);
begin
if buLoop.caption<>'Start Loop' then begin
buLoop.caption:='Start Loop';
Timer1.enabled:=false;
end // no ; here
else begin
buLoop.caption:='Stop Loop';
Timer1.enabled:=true;
end;
end;

procedure TDS030f1.Timer1Timer(Sender: TObject);
begin
inc(bCycles);
if bCycles>254 then bCycles:=0;
if bCycles mod 2=0 then begin
buReadPIOClick(buQuit);(*The buQuit parameter just
fills a need. Nothing is done with it*)
end//no ; here
else buWriteToOutputClick(buQuit);
end;

procedure TDS030f1.buSafeClick(Sender: TObject);
begin
showmessage('See helpfile. You must not enable a channel which '+
'can, through external circuitry, be connected directly to 5v, '+
'or to something which would result in a large current.');
end;

procedure TDS030f1.cbPIOEnabled7Click(Sender: TObject);
begin
if cbPIOEnabled7.checked then cbOutput7.enabled:=true // no ;
else cbOutput7.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled6Click(Sender: TObject);
begin
if cbPIOEnabled6.checked then cbOutput6.enabled:=true // no ;
else cbOutput6.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled5Click(Sender: TObject);
begin
if cbPIOEnabled5.checked then cbOutput5.enabled:=true // no ;
else cbOutput5.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled4Click(Sender: TObject);
begin
if cbPIOEnabled4.checked then cbOutput4.enabled:=true // no ;
else cbOutput4.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled3Click(Sender: TObject);
begin
if cbPIOEnabled3.checked then cbOutput3.enabled:=true // no ;
else cbOutput3.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled2Click(Sender: TObject);
begin
if cbPIOEnabled2.checked then cbOutput2.enabled:=true // no ;
else cbOutput2.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled1Click(Sender: TObject);
begin
if cbPIOEnabled1.checked then cbOutput1.enabled:=true // no ;
else cbOutput1.enabled:=false;
end;

procedure TDS030f1.cbPIOEnabled0Click(Sender: TObject);
begin
if cbPIOEnabled0.checked then cbOutput0.enabled:=true // no ;
else cbOutput0.enabled:=false;
end;

procedure TDS030f1.DMLInit;
var c1:byte;
begin
(**You need to change these things before the program will
   run with your hardware. Search target ref'd in DS030.hlp:
       qhelptarget.
The Dallas id of your chip needs to go into the following string
array element. iButtonViewer will tell you the id, even if it
won't "drive" the chip. Enter the ID with the family code ($29)
at the right hand end. There should be one character for each
"v" in the following aid to accuracy....*)
(*              vvvvvvvvvvvvvv29*)
saDMLRomID[0]:='CB000000000A6929';
//CB000000000A6929 TAI8558 from Aitor 5/04
//F000000000448A29 chip in pcb199a
(*Prepare two variables with the facts of your
adapter location and type. There are fancier ways
to do this, and the information can be auto-
detected. This stuff should at least be supplied
to the program as command line parameters in a
non-demo program*)
iDMLPortNum:=1;(*Which serial port your adapter is on*)
iDMLAdapterType:=5;(*Adapter types...
   1:DS9097 with no suffix, 9097E
   5:DS9097U, 9097U-9*)
(*Put the IDs of your Dallas chips into
  the elements of saDMLRomID. You can
  learn your chips' IDs with the iButton viewer.
  Note IDs given with family ID at right hand end.*)
bDMLRomIndexLast:=0;
for c1:=0 to bDMLRomIndexLast do begin
 sTmp:=saDMLRomID[c1];
 baDMLRomID[c1,7]:=strtoint('$'+copy(sTmp,1,2));
 baDMLRomID[c1,6]:=strtoint('$'+copy(sTmp,3,2));
 baDMLRomID[c1,5]:=strtoint('$'+copy(sTmp,5,2));
 baDMLRomID[c1,4]:=strtoint('$'+copy(sTmp,7,2));
 baDMLRomID[c1,3]:=strtoint('$'+copy(sTmp,9,2));
 baDMLRomID[c1,2]:=strtoint('$'+copy(sTmp,11,2));
 baDMLRomID[c1,1]:=strtoint('$'+copy(sTmp,13,2));
 baDMLRomID[c1,0]:=strtoint('$'+copy(sTmp,15,2));
  end;(*For...*)
end;(*DMLInit*)

procedure TDS030f1.DMLGetPIOLogicState(bIndex:byte;
         var bAns:byte;
         var sTKBErr:string;
         var iDalErr:integer);
(*In some Dallas demo code, there seems to be
  places where the code says
  "Couldn't do that first time? Try a few times more"
  In this code, we work through the stages of reading
  a temperature, and if we get a snag at any point,
  we give up. It is up to the calling program to
  re-call this as often as it deems worthwhile.

  N.B. This code does not check that the device it has
  been sent to read is in fact a DS2408 (family code $29).
  That check should be made- elsewhere.*)
var c1:byte;

begin
sTKBErr:='0';
bAns:=0;

liDMLSHandle:=TMExtendedStartSession(iDMLPortNum,
                 iDMLAdapterType,nil);
if liDMLSHandle<1 then begin
  sTKBErr:='10DML2407GPLS';
  iDalErr:=liDMLSHandle;
  end;

if sTKBErr='0' then begin
  iDMLTmpResult:=TMSetUp(liDMLSHandle);
  if iDMLTmpResult<>1 then begin
    sTKBErr:='20DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;

//Reset/Presence/Skip ROM
if sTKBErr='0' then begin
  for c1:=0 to 7 do siaDMLRom[c1]:=baDMLRomID[bIndex,c1];
  (*Prepare to pass chip ID to a buffer in TMEX driver.
    It MAY not always be necessary to repeatedly refill
    this buffer, but doing so makes this code more generally
    useful.*)
  try (*You could introduce other try... except... end blocks.
        I only put them in where I needed them to work past
        various errors in my code, errors which I hope are
        now gone!*)
  iDMLTmpResult:=TMRom(liDMLSHandle,@baDMLStateBuffer,@siaDMLRom);
  except
  (*DID WORK.. REIMPLEMENT...on EGPFault do begin
          sTKBErr:='31';
          iDalErr:=0;
          iDMLTmpResult:=1;
          end;              *)
  end;(*Except*)
  if iDMLTmpResult<>1 then begin
    sTKBErr:='30DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;

if sTKBErr='0' (*CSess*) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
(*Check to see that liDMLSHandle, the session handle, is still
    valid*)
if iDMLTmpResult<>1 then begin
    sTKBErr:='32DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
end;
(*Here begins: Turn off all devices apart from the
  one you want...*)
if sTKBErr='0' then begin
  iDMLTmpResult:=TMStrongAccess(liDMLSHandle,@baDMLStateBuffer);
  if iDMLTmpResult<>1 then begin
    sTKBErr:='40DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='42DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;
*)

(*Issue Read PIO Registers Cmnd*)
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,$F0);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2407GPLS';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

(*Issue first byte of required register (PIO Logic State*)
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,$88);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2407GPLS';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

(*Issue second byte of required register*)
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,$00);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2407GPLS';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

//Read data from the register
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,$FF);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;
(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

//Read the nine data..
(*if sTKBErr='0' then begin
  iD0:=TMTouchByte(liDMLSHandle,$FF);
  if iD0<0 then begin
    sTKBErr:='70DML2407GPLS';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML)
    end;
  end;
(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2407GPLS';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

bAns:=iDMLTmpResult;
end; //DMLGetPIOLogicState

procedure TDS030f1.DMLWriteToEnabledOutputs
    (bIndex,bValue,bMask:byte;
    var sTKBErr:string;
    var iDalErr:integer);
var c1:byte;
(*Writes to device bIndex what is in bValue, but masked
  with what's in bMask.
  ****If a i/o line can, due to external circuitry, be connected
  directly to 5v, then that line must not be set as an output
  to zero.
  In parentheses: MY CONJECTURE>>>NOT NECESSAR|ILY TRUE!!!>>>
  (Connecting a "set high" output to 0v externally
  does no harm, due to the chip's internals... and this also
  means that trying to use the chip to source significant current
  is a waste of time.)****
  A 1 in a bit of bMask enables setting that bit to 0 as an output.
  The chip does not have data direction registers, so,
  following comment: MY CONJECTURE>>>NOT NECESSARILY TRUE!!!>>>
  if a bit is set as an output of "1", it is largely meaningless
  and ineffective, beyond being "not 0". A simple illustration:
  The outputs can sink sufficient current to drive an LED. Connect
  from 5v to the LED, thence to a 470 ohm resistor, thence to a
  pin of the DS2408. If the chip drives the pin low, the LED will
  glow (if you inserted it the right way 'round. If not, no harm
  done, just reverse the LED. If the chip drives the pin "high",
  the chip won't glow. However, reversing the circuit, so that
  the electricity would flow from the DS2408, through the LED
  and resistor, will not work... the chip won't source enough
  current.
  N.B.: The data sheet quote the pins' performance at a current of
  4mA, so I would infer that this is a "typical" operating level.
  It also shows LEDs being driven as described, at a current of
  8mA, so that would seem to be acceptable. The TOTAL current through
  the IO pins is not meant to exceed 20mA, so DON'T ATTACH 8 LEDs
  directly. If you want more than 2, you need to use transistors.
  (Not hard, just an extra bit if circuit to build.)
  *)
begin
bValue:=bValue or (not bMask);
(*The above makes a 1 out of any 0 in bValue which is in the same
 position as a 0 in bMask. This means that only bits ENabled by
 ONEs in bMask will be set to 0's by 0's in bValue. The slight
 complexity introduced with the "not" is there so that the mask
 byte can have 1's in enabled positons, which is a little more
 user friendly than alternate schemes.*)
//showmessage('$'+inttohex(bValue,2));
sTKBErr:='0';

liDMLSHandle:=TMExtendedStartSession(iDMLPortNum,
                 iDMLAdapterType,nil);
if liDMLSHandle<1 then begin
  sTKBErr:='11DML2408WTEO';
  iDalErr:=liDMLSHandle;
  end;

if sTKBErr='0' then begin
  iDMLTmpResult:=TMSetUp(liDMLSHandle);
  if iDMLTmpResult<>1 then begin
    sTKBErr:='20DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;

//Reset/Presence/Skip ROM
if sTKBErr='0' then begin
  for c1:=0 to 7 do siaDMLRom[c1]:=baDMLRomID[bIndex,c1];
  (*Prepare to pass chip ID to a buffer in TMEX driver.
    It MAY not always be necessary to repeatedly refill
    this buffer, but doing so makes this code more generally
    useful.*)
  try (*You could introduce other try... except... end blocks.
        I only put them in where I needed them to work past
        various errors in my code, errors which I hope are
        now gone!*)
  iDMLTmpResult:=TMRom(liDMLSHandle,@baDMLStateBuffer,@siaDMLRom);
  except
  (*DID WORK.. REIMPLEMENT...on EGPFault do begin
          sTKBErr:='31DML2408WTEO';
          iDalErr:=0;
          iDMLTmpResult:=1;
          end;              *)
  end;(*Except*)
  if iDMLTmpResult<>1 then begin
    sTKBErr:='30DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;

if sTKBErr='0' (*CSess*) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
(*Check to see that liDMLSHandle, the session handle, is still
    valid*)
if iDMLTmpResult<>1 then begin
    sTKBErr:='32DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
end;
(*Here begins: Turn off all devices apart from the
  one you want...*)
if sTKBErr='0' then begin
  iDMLTmpResult:=TMStrongAccess(liDMLSHandle,@baDMLStateBuffer);
  if iDMLTmpResult<>1 then begin
    sTKBErr:='40DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='42DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;
*)

(*Issue Channel-Access Write Cmnd*)
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,$5A);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2408WTEO';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

(*Write the data (to register $89 (impicit in comnd))
  Remember: bValue has been massaged so that 1's are
  written to not-enabled bits.*)
  //showmessage(inttostr(bValue));
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,bValue);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2408WTEO';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

(*Now write the data (to register $89 (impicit in comnd),
  but inverted, as required by protocol.
  Remember: bValue has been massaged so that 1's are
  written to not-enabled bits.*)
  //showmessage(inttostr(bValue));
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,not(bValue));
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2408WTEO';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
    end;
  end;

(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)

//Channel-Access Write protocol provides for read-after-write...
//of which this program does not avail itself.

//...if using read-after-write....

(*
//Read data from the register
if sTKBErr='0' then begin
  iDMLTmpResult:=TMTouchByte(liDMLSHandle,$FF);
  if iDMLTmpResult<0 then begin
    sTKBErr:='70DML2408WTEO';
    iDalErr:=iDMLTmpResult;(*-1 flags shorted ML)
    end;
  end;
(*Could? Should? CheckSess occur here?
if sTKBErr='0' (*CSess) then begin
        iDMLTmpResult:=TMValidSession(liDMLSHandle);
if iDMLTmpResult<>1 then begin
    sTKBErr:='52DML2408WTEO';
    iDalErr:=iDMLTmpResult;
    end;
  end;*)


end;//DMLWriteToEnabledOutputs

end.
